Type TGameObject
	Global tlallGameObjects:TList = New TList
	Global iObjectCount:Int = 11
	Global sPath:String = "Data/gfx/Objects/"
	Field v2dMapPosition:TVector2d
	Field iOwnIdentNumber:Int
	Field tdiDrawnImage:TDrawnImage
	
	Function UpdateAll()
		For Local Obj:TGameObject = EachIn tlallGameObjects
			Obj.Update()
		Next
	End Function
	
	Function INITAll()
		TKey.INIT()
		TFireItem.Init()
		TWaterItem.INIT()
		TEarthItem.Init()
		TWindItem.Init()
		TExit.init()
		TFireWall.INIT()
		TWaterWall.INIT()
		TEarthWall.INIT()
		TWindWall.INIT()
	End Function
	
	Function DrawAll()
		For Local Obj:TGameObject = EachIn tlallGameObjects
			Obj.Draw()
		Next
	End Function
	
	Function GetExit:TExit()
		For Local Obj:TGameObject = EachIn tlallGameObjects
			If Obj.iOwnIdentNumber = TExit.iIDENTNUMBER Then
				Return TExit(Obj)
			End If
		Next
	End Function
	
	Function GetPlayerStart:TVector2D()
		For Local Obj:TGameObject = EachIn tlallGameObjects
			If Obj.iOwnIdentNumber = TPlayerStart.iIDENTNUMBER Then
				Return Obj.v2dMapPosition
			End If
		Next
		Return Null
	End Function
	
	Function DeleteObject(pv2dPosition:TVector2D, piValue:Int)
		For Local Obj:TGameObject = EachIn tlallGameObjects
			If (Int(Obj.v2dMapPosition.Get_X()) = Int(pv2dPosition.Get_X())) Then
				If (Int(Obj.v2dMapPosition.Get_y()) = Int(pv2dPosition.Get_y())) Then
					If Obj.iOwnIdentNumber = piValue Then
						Obj.Destroy()
						TGameObjectsProvider.gmMap.ChangeTileValue(TGameMap.iOBJECTLAYER,  ..
															pv2dPosition)
					EndIf
				EndIf
			EndIf
		Next
	End Function
	
	Function SetNewObject(pv2dPosition:TVector2D, piValue:Int)
		Select piValue
			Case 1
				TExit.Create(pv2dPosition)
			Case 2
				TFireWall.Create(pv2dPosition)
			Case 3
				TWaterWall.Create(pv2dPosition)
			Case 4
				TEarthWall.Create(pv2dPosition)
			Case 5
				TWindWall.Create(pv2dPosition)
			Case 6
				TFireItem.Create(pv2dPosition)
			Case 7
				TWaterItem.Create(pv2dPosition)
			Case 8
				TEarthItem.Create(pv2dPosition)
			Case 9
				TWindItem.Create(pv2dPosition)
			Case 10
				TPlayerStart.Create(pv2dPosition)
			Case 11
				TKey.Create(pv2dPosition)
		End Select
	End Function
	
	Function LoadObject(sStream:TStream, piValue:Int)
		Select piValue
			Case 1
				TExit.LoadFromFile(sStream)
			Case 2
				TFireWall.LoadFromFile(sStream)
			Case 3
				TWaterWall.LoadFromFile(sStream)
			Case 4
				TEarthWall.LoadFromFile(sStream)
			Case 5
				TWindWall.LoadFromFile(sStream)
			Case 6
				TFireItem.LoadFromFile(sStream)
			Case 7
				TWaterItem.LoadFromFile(sStream)
			Case 8
				TEarthItem.LoadFromFile(sStream)
			Case 9
				TWindItem.LoadFromFile(sStream)
			Case 10
				TPlayerStart.LoadFromFile(sStream)
			Case 11
				TKey.LoadFromFile(sStream)
		End Select
	End Function
	
	Function DrawInEditor(pv2dPosition:TVector2D, piValue:Int)
		Select piValue
			Case 1
				TExit.DrawForEditor(pv2dPosition)
			Case 2
				TFireWall.DrawForEditor(pv2dPosition)
			Case 3
				TWaterWall.DrawForEditor(pv2dPosition)
			Case 4
				TEarthWall.DrawForEditor(pv2dPosition)
			Case 5
				TWindWall.DrawForEditor(pv2dPosition)
			Case 6
				TFireItem.DrawForEditor(pv2dPosition)
			Case 7
				TWaterItem.DrawForEditor(pv2dPosition)
			Case 8
				TEarthItem.DrawForEditor(pv2dPosition)
			Case 9
				TWindItem.DrawForEditor(pv2dPosition)
			Case 10
				TPlayerStart.DrawForEditor(pv2dPosition)
			Case 11
				TKey.DrawForEditor(pv2dPosition)
		End Select
	End Function
	
	Function DestroyAll()
		tlallGameObjects.Clear()
	End Function
	
	Method New()
		tlallGameObjects.AddLast(Self)
	End Method
	
	Method Destroy()
		tlallGameObjects.Remove(Self)
	End Method
	
	Method SaveToFile(sStream:TStream)
		WriteInt(sStream, iOwnIdentNumber)
		WriteInt(sStream, v2dMapPosition.Get_X())
		WriteInt(sStream, v2dMapPosition.Get_Y())
	End Method
	
	Method Update()
		
	End Method
	
	Method Draw()
		
	End Method
End Type

Type TExit Extends TGameObject
	Const iIDENTNUMBER:Int = 1
	Const sImageName:String = "ExitImage"
	
	Field bIsOpen:Byte
	
	Function Create:TExit(pv2dPos:TVector2D)
		Local Ex:TExit = New TExit
		
		Ex.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		Ex.bIsOpen = False
		Ex.iOwnIdentNumber = iIDENTNUMBER
		Ex.MakeSureIsOnlyOne()
		Ex.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return Ex
	End Function
	
	Function INIT()
		TLoadedImage.CreateAnimated(sPath + sImageName + ".png", sImageName, 32, 32, 2, 0)
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TExit.Create(TVector2D.Create(x, y))
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				If TGameObjectsProvider.tpPlayer.bHasKey Then
					TGameObjectsProvider.LoadNextMap()
				EndIf
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TFireWall Extends TGameObject
	Const iIDENTNUMBER:Int = 2
	Const sImageName:String = "FireWall"
	
	Function Create:TFireWall(pv2dPos:TVector2D)
		Local FWall:TFireWall = New TFireWall
		
		FWall.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		FWall.iOwnIdentNumber = iIDENTNUMBER
		TGameObjectsProvider.gmMap.iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 1
		FWall.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return FWall
	End Function
	
	Function INIT()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Method Destroy()
		Super.Destroy()
		TGameObjectsProvider.gmMap.iaMapArray[v2dMapPosition.Get_X(), v2dMapPosition.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 0
	End Method
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TFireWall.Create(TVector2D.Create(x, y))
	End Function

	Method Update()
		If TGameObjectsProvider.tpPlayer.bHasWater Then
			Self.Destroy()
		EndIf
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TWaterWall Extends TGameObject
	Const iIDENTNUMBER:Int = 3
	Const sImageName:String = "WaterWall"
	
	Function Create:TWaterWall(pv2dPos:TVector2D)
		Local WWall:TWaterWall = New TWaterWall
		
		WWall.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		WWall.iOwnIdentNumber = iIDENTNUMBER
		TGameObjectsProvider.gmMap.iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 1
		WWall.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return WWall
	End Function
	
	Function INIT()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Method Destroy()
		Super.Destroy()
		TGameObjectsProvider.gmMap.iaMapArray[v2dMapPosition.Get_X(), v2dMapPosition.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 0
	End Method
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TWaterWall.Create(TVector2D.Create(x, y))
	End Function

	Method Update()
		If TGameObjectsProvider.tpPlayer.bHasEarth Then
			Self.Destroy()
		EndIf
	End Method
	
		Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TEarthWall Extends TGameObject
	Const iIDENTNUMBER:Int = 4
	Const sImageName:String = "EarthWall"
	
	Function Create:TEarthWall(pv2dPos:TVector2D)
		Local EWall:TEarthWall = New TEarthwall
		
		EWall.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		EWall.iOwnIdentNumber = iIDENTNUMBER
		TGameObjectsProvider.gmMap.iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 1
		EWall.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return EWall
	End Function
	
	Function INIT()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Method Destroy()
		Super.Destroy()
		TGameObjectsProvider.gmMap.iaMapArray[v2dMapPosition.Get_X(), v2dMapPosition.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 0
	End Method
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TEarthWall.Create(TVector2D.Create(x, y))
	End Function

	Method Update()
		If TGameObjectsProvider.tpPlayer.bHasWind Then
			Self.Destroy()
		EndIf
	End Method
	
		Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TWindWall Extends TGameObject
	Const iIDENTNUMBER:Int = 5
	Const sImageName:String = "WindWall"
	
	Function Create:TWindWall(pv2dPos:TVector2D)
		Local WiWall:TWindWall = New TWindWall
		
		WiWall.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		WiWall.iOwnIdentNumber = iIDENTNUMBER
		TGameObjectsProvider.gmMap.iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 1
		WiWall.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return WiWall
	End Function
	
	Function INIT()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Method Destroy()
		Super.Destroy()
		TGameObjectsProvider.gmMap.iaMapArray[v2dMapPosition.Get_X(), v2dMapPosition.Get_Y(),  ..
											 TGameMap.iCOLLISIONLAYER] = 0
	End Method
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TWindWall.Create(TVector2D.Create(x, y))
	End Function

	Method Update()
		If TGameObjectsProvider.tpPlayer.bHasFire Then
			Self.Destroy()
		EndIf
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TFireItem Extends TGameObject
	Const iIDENTNUMBER:Int = 6
	Const sImageName:String = "FireItem"
	
	Function Create:TFireItem(pv2dPos:TVector2D)
		Local FItem:TFireItem = New TFireItem
		
		FItem.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		FItem.iOwnIdentNumber = iIDENTNUMBER
		FItem.MakeSureIsOnlyOne()
		FItem.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))
		
		Return FItem
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TFireItem.Create(TVector2D.Create(x, y))
	End Function
	
	Function Init()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				Self.Destroy()
				TGameObjectsProvider.tpPlayer.bHasFire = True
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TWaterItem Extends TGameObject
	Const iIDENTNUMBER:Int = 7
	Const sImageName:String = "WaterItem"
	
	Function Create:TWaterItem(pv2dPos:TVector2D)
		Local WItem:TWaterItem = New TWaterItem
		
		WItem.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		WItem.iOwnIdentNumber = iIDENTNUMBER
		WItem.MakeSureIsOnlyOne()
		WItem.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))
		
		Return WItem
	End Function
	
	Function Init()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TWaterItem.Create(TVector2D.Create(x, y))
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				Self.Destroy()
				TGameObjectsProvider.tpPlayer.bHasWater = True
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TEarthItem Extends TGameObject
	Const iIDENTNUMBER:Int = 8
	Const sImageName:String = "EarthItem"
	
	Function Create:TEarthItem(pv2dPos:TVector2D)
		Local EItem:TEarthItem = New TEarthItem
		
		EItem.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		EItem.iOwnIdentNumber = iIDENTNUMBER
		EItem.MakeSureIsOnlyOne()
		EItem.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return EItem
	End Function
	
	Function Init()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TEarthItem.Create(TVector2D.Create(x, y))
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				Self.Destroy()
				TGameObjectsProvider.tpPlayer.bHasEarth = True
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TWindItem Extends TGameObject
	Const iIDENTNUMBER:Int = 9
	Const sImageName:String = "WindItem"
	
	Function Create:TWindItem(pv2dPos:TVector2D)
		Local WiItem:TWindItem = New TWindItem
		
		WiItem.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		WiItem.iOwnIdentNumber = iIDENTNUMBER
		WiItem.MakeSureIsOnlyOne()
		WiItem.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage(sImageName))

		Return WiItem
	End Function
	
	Function Init()
		TLoadedImage.Create(sPath + sImageName + ".png", sImageName)
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TWindItem.Create(TVector2D.Create(x, y))
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				Self.Destroy()
				TGameObjectsProvider.tpPlayer.bHasWind = True
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage(sImageName).Draw(pv2dposition)
	End Function
End Type

Type TPlayerStart Extends TGameObject
	Const iIDENTNUMBER:Int = 10
	
	Function Create:TPlayerStart(pv2dPos:TVector2D)
		Local PStart:TPlayerStart = New TPlayerStart
		
		PStart.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		PStart.iOwnIdentNumber = iIDENTNUMBER
		PStart.MakeSureIsOnlyOne()

		Return PStart
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TPlayerStart.Create(TVector2D.Create(x, y))
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()

	End Method
	
	Method Draw()
'		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
'		SetColor(150, 255, 100)
'		DrawOval(RealV2d.Get_X() + 5, RealV2d.get_Y() + 5, 20, 20)
'		SetColor(255, 255, 255)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		SetColor(150, 255, 100)
		DrawOval(pv2dPosition.get_X() - 10, pv2dPosition.Get_Y() - 10, 20, 20)
		SetColor(255, 255, 255)
	End Function
End Type

Type TKey Extends TGameObject
	Const iIDENTNUMBER:Int = 11
	
	Function Create:TKey(pv2dPos:TVector2D)
		Local Key:TKey = New TKey
		
		Key.v2dMapPosition = TVector2D.Create(pv2dPos.Get_X(), pv2dPos.Get_Y())
		Key.iOwnIdentNumber = iIDENTNUMBER
		Key.MakeSureIsOnlyOne()
		Key.tdiDrawnImage = TDrawnImage.Create(TLoadedImage.GetImage("Key"))

		Return Key
	End Function
	
	Function LoadFromFile(sStream:TStream)
		Local x:Int, y:Int
		x = ReadInt(sStream)
		y = ReadInt(sStream)
		TKey.Create(TVector2D.Create(x, y))
	End Function
	
	Function INIT()
		TLoadedImage.Create("Data/gfx/Objects/Key.png", "Key")
	End Function
	
	Method MakeSureIsOnlyOne()
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			If Obj <> Self Then
				If Obj.iOwnIdentNumber = iOwnIdentNumber Then
					TGameObject.tlallGameObjects.Remove(obj)
				End If
			End If
		Next
	End Method
	
	Method Update()
		If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_X() = v2dMapPosition.Get_X() Then
			If TGameObjectsProvider.tpPlayer.v2dPositionMap.Get_y() = v2dMapPosition.Get_y() Then
				Self.Destroy()
				TGameObjectsProvider.tpPlayer.bHasKey = True
				TExit(TGameObject.GetExit()).tdiDrawnImage.AnimateManual()
			End If
		End If
	End Method
	
	Method Draw()
		Local RealV2d:TVector2D = TGamemap.CalculateRealPosition(v2dmapPosition)
		RealV2d.Set_X(RealV2d.Get_X() + TGameMap.iTILESIZE / 2)
		RealV2d.Set_y(RealV2d.Get_y() + TGameMap.iTILESIZE / 2)
		tdiDrawnImage.Draw(RealV2d)
	End Method
	
	Function DrawForEditor(pv2dPosition:TVector2D)
		TLoadedImage.GetImage("Key").Draw(pv2dposition)
	End Function
End Type