Type TGameMap
	Const iLAYERCOUNT:Int = 3
	Const iCOLLISIONLAYER:Int = 0
	Const iTILELAYER:Int = 1
	Const iOBJECTLAYER:Int = 2
	Const iTILESIZE:Int = 32
	Const sFILEEXTENSION:String = ".mmp"
	
	Field v2dMapSize:TVector2D
	Field bIsDark:Byte
	Field iaMapArray:Int[,,]
	
	Function Create:TGameMap(pv2dSize:TVector2D)
		Local Map:TGameMap = New TGameMap
		
		Map.v2dMapSize = TVector2D.Create(pv2dSize.Get_X(), pv2dSize.Get_Y())
		Map.iaMapArray = New Int[Int(pv2dSize.Get_X()), Int(pv2dSize.Get_Y()), iLAYERCOUNT]
		Map.ClearGameMap()
		Map.bIsDark = False
		
		Return Map
	End Function
	
	Function CreateLoaded:TGameMap(pv2dSize:TVector2D, piaMapArray:Int[,,], pbIsDark:Byte)
		Local Map:TGameMap = New TGameMap
		
		Map.v2dMapSize = TVector2D.Create(pv2dSize.Get_X(), pv2dSize.Get_Y())
		Map.iaMapArray = New Int[Int(pv2dSize.Get_X()), Int(pv2dSize.Get_Y()), iLAYERCOUNT]
		Map.FillFromOtherArray(piaMapArray)
		map.bIsDark = pbIsDark
		
		Return Map
	End Function
	
	Method ClearGameMap()
		For Local iLAYERCOUNT:Int = 0 Until iLAYERCOUNT
			For Local iX:Int = 0 Until v2dMapSize.Get_X()
				For Local iY:Int = 0 Until v2dMapSize.Get_Y()
					iaMapArray[ix, iy, iLAYERCOUNT] = 0
				Next
			Next
		Next
	End Method
	
	Method FillFromOtherArray(piaMapArray:Int[,,])
		For Local iX:Int = 0 Until v2dMapSize.Get_X()
			For Local iY:Int = 0 Until v2dMapSize.Get_Y()
				iaMapArray[iX, iY, iTILELAYER] = piaMapArray[iX, iY, iTILELAYER]
				iaMapArray[iX, iY, iCOLLISIONLAYER] = piaMapArray[iX, iY, iCOLLISIONLAYER]
				iaMapArray[iX, iY, iOBJECTLAYER] = piaMapArray[iX, iY, iOBJECTLAYER]
			Next
		Next
	End Method
	
	Method IsInRange:Byte(pv2dPosition:TVector2D)
		Local bIsInRange:Byte = (Int(pv2dPosition.Get_X()) < v2dMapSize.Get_X())
		bIsInRange = bIsInRange And (Int(pv2dPosition.Get_Y() < v2dMapSize.Get_Y()))
		bIsInRange = bIsInRange And (Int(pv2dPosition.Get_X() >= 0))
		bIsInRange = bIsInRange And (Int(pv2dPosition.Get_Y() >= 0))
		Return bIsInRange
	End Method
	
	Method IsWalkable:Byte(pv2dPosition:TVector2D)
		If IsInRange(pv2dPosition) Then
			Return (iaMapArray[Int(pv2dPosition.Get_X()),  ..
					 Int(pv2dPosition.Get_Y()), iCOLLISIONLAYER] = 0)
		EndIf
		Return False
	End Method
	
	Method DrawMap(pbDrawCollision:Byte = False)
		For Local iX:Int = 0 Until v2dMapSize.Get_X()
			For Local iY:Int = 0 Until v2dMapSize.Get_Y()
				Local iValue:Int = iaMapArray[ix, iy, iTileLayer]
				Local v2dPos:TVector2D = TVector2d.Create(ix * itilesize + itilesize / 2,  ..
														 iY * iTILESIZE + iTILESIZE / 2)
				TGameObjectsProvider.tsTileSet.Draw(iValue, v2dPos)
'				If iX = (v2dMapSize.get_X() - 1) Then
'					If iY = (v2dMapSize.get_Y() - 1) Then
'						Local sString:String = "LL"
'						DrawText(sString, iX * iTILESIZE + iTILESIZE / 2 - TextWidth(sString) / 2,  ..
'								iY * iTILESIZE + iTILESIZE / 2 - TextHeight(sString) / 2)
'					EndIf
'				EndIf
'				If iX = 0 Then
'					If iY = 0 Then
'						Local sString:String = "00"
'						DrawText(sString, iX * iTILESIZE + iTILESIZE / 2 - TextWidth(sString) / 2,  ..
'								iY * iTILESIZE + iTILESIZE / 2 - TextHeight(sString) / 2)
'					End If
'				End If
				If pbDrawCollision Then
					If iaMapArray[iX, iY, iCOLLISIONLAYER] = 1 Then
						SetAlpha(0.5)
						v2dPos = TVector2D.Create(iX * iTILESIZE,  ..
													iY * iTILESIZE)
						DrawRect(v2dPos.Get_X(), v2dPos.Get_Y(), iTILESIZE, iTILESIZE)
						SetAlpha(1)
					EndIf
				EndIf
			Next
		Next
		SetColor(255, 255, 255)
	End Method
	
	Method ChangeTileValue(piLayer:Int, pv2dPos:TVector2D, piLayerValue:Int = 0, piObjectValue:Int = 0)
		If IsInRange(pv2dPos) Then
			Select piLayer
				Case iCOLLISIONLAYER
					ChangeCollision(pv2dPos)
				Case iTILELAYER
					ChangeTile(pv2dPos, piLayerValue)
				Case iOBJECTLAYER
					ChangeObject(pv2dPos, piObjectValue)
			End Select
		EndIf
	End Method
	
	Method ChangeCollision(pv2dPos:TVector2D)
		iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(), iCOLLISIONLAYER] = Not iaMapArray[pv2dPos.Get_X(),  ..
																	 				pv2dPos.Get_Y(),  ..
																	 				iCOLLISIONLAYER]
	End Method
	
	Method ChangeObject(pv2dPos:TVector2D, piValue:Int)
		If (iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(), iOBJECTLAYER] <> piValue) Then
			TGameObject.DeleteObject(pv2dPos,  ..
					 iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(), iOBJECTLAYER])
			iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(), iOBJECTLAYER] = piValue
			TGameObject.SetNewObject(pv2dPos, piValue)
		EndIf
	End Method
	
	Method SaveMap(psPath:String)
		Local tsFile:TStream = WriteFile(psPath)
		WriteInt(tsFile, v2dMapSize.Get_x())
		WriteInt(tsFile, v2dMapSize.Get_Y())
		WriteByte(tsFile, bIsDark)
		For Local iX:Int = 0 Until v2dMapSize.Get_X()
			For Local iY:Int = 0 Until v2dMapSize.Get_Y()
				WriteInt(tsFile, iaMapArray[iX, iY, iCOLLISIONLAYER])
			Next
		Next
		For Local iX:Int = 0 Until v2dMapSize.Get_X()
			For Local iY:Int = 0 Until v2dMapSize.Get_Y()
				WriteInt(tsFile, iaMapArray[iX, iY, iTILELAYER])
			Next
		Next
		For Local iX:Int = 0 Until v2dMapSize.Get_X()
			For Local iY:Int = 0 Until v2dMapSize.Get_Y()
				WriteInt(tsFile, iaMapArray[iX, iY, iOBJECTLAYER])
			Next
		Next
		For Local Obj:TGameObject = EachIn TGameObject.tlallGameObjects
			Obj.SaveToFile(tsFile)
		Next
		CloseFile(tsFile)
	End Method
	
	Function LoadMap:TGameMap(psPath:String)
		Local v2dSize:TVector2D = TVector2d.Zero()
		Local iaNewArray:Int[,,]
		Local bnewIsDark:Byte
		If FileType(psPath) = 1 Then
			Local tsFile:TStream = ReadFile(psPath)
			v2dSize.Set_X(ReadInt(tsFile))
			v2dSize.Set_Y(ReadInt(tsFile))
			bnewIsDark = ReadByte(tsFile)
			iaNewArray = New Int[Int(v2dSize.Get_X()), Int(v2dSize.Get_Y()), iLAYERCOUNT]
			For Local ix:Int = 0 Until v2dSize.Get_X()
				For Local iY:Int = 0 Until v2dSize.Get_Y()
					iaNewArray[ix, iY, iCOLLISIONLAYER] = ReadInt(tsFile)
				Next
			Next
			For Local ix:Int = 0 Until v2dSize.Get_X()
				For Local iY:Int = 0 Until v2dSize.Get_Y()
					iaNewArray[ix, iY, iTILELAYER] = ReadInt(tsFile)
				Next
			Next
			For Local ix:Int = 0 Until v2dSize.Get_X()
				For Local iY:Int = 0 Until v2dSize.Get_Y()
					iaNewArray[ix, iY, iOBJECTLAYER] = ReadInt(tsFile)
				Next
			Next
			If Not Eof(tsFile) Then
				Repeat
					Local iIdent:Int = ReadInt(tsfile)
					TGameObject.LoadObject(tsFile, iIdent)
				Until Eof(tsFile)
			EndIf
			Return TGameMap.CreateLoaded(v2dSize, iaNewArray, bnewIsDark)
		EndIf
	End Function
	
	Method ChangeTile(pv2dPos:TVector2D, piValue:Int)
		iaMapArray[pv2dPos.Get_X(), pv2dPos.Get_Y(), iTILELAYER] = piValue
		iaMapArray[pv2dPos.get_X(), pv2dPos.get_Y(), iCOLLISIONLAYER] = piValue
	End Method
	
	Function CalculateMapPosition:TVector2D(pv2dPosition:TVector2D)
		Return TVector2D.Create(Int(pv2dPosition.Get_X() / iTILESIZE),  ..
							 Int(pv2dPosition.Get_Y() / iTILESIZE))
	End Function
	
	Function CalculateRealPosition:TVector2D(pv2dPosition:TVector2D)
		Return TVector2D.Create(pv2dPosition.Get_X() * iTILESIZE,  ..
							 pv2dPosition.Get_Y() * iTILESIZE)
	End Function
End Type

